@@ -5,8 +5,8 @@ class ConectividadeAppConfig(PluginConfig):
     name = 'conectividadeapp'
     verbose_name = 'conectividade'
     description = 'An plugin'
-    version = '1.1.0'
-    author = 'Johnny; Avando; Eduardo; Rary'
+    version = '1.1.2'
+    author = 'Johnny; Avando; Eduardo; Rary; Thiago'
     author_email = 'johnnyyuri56@gmail.com'
     required_settings = []
     default_settings = {
