@@ -2,7 +2,7 @@ from django.contrib import admin
 
 from .models import Actor, ActorCategory
 from .models import OldDevice
-from .models import Activity, ActivityInstall, ActivityRemove, ActivityReason
+from .models import ActivityReason, Activity
 
 @admin.register(ActorCategory)
 class ActorCategoryAdmin(admin.ModelAdmin):
@@ -20,15 +20,15 @@ class OldDevice(admin.ModelAdmin):
 
 @admin.register(Activity)
 class ActivityAdmin(admin.ModelAdmin):
-    list_display = ['id', 'description', 'type']
+    list_display = ['id', 'when', 'description','reason' , 'type']
 
-@admin.register(ActivityInstall)
-class ActivityInstallAdmin(admin.ModelAdmin):
-    list_display = ['id', 'when', 'description', 'reason', 'type']
-
-@admin.register(ActivityRemove)
-class ActivityRemoveAdmin(admin.ModelAdmin):
-    list_display = ['id', 'when', 'description', 'reason', 'type']
+# @admin.register(ActivityInstall)
+# class ActivityInstallAdmin(admin.ModelAdmin):
+#     list_display = ['id', 'when', 'description', 'reason', 'type']
+#
+# @admin.register(ActivityRemove)
+# class ActivityRemoveAdmin(admin.ModelAdmin):
+#     list_display = ['id', 'when', 'description', 'reason', 'type']
 
 @admin.register(ActivityReason)
 class ActivityReason(admin.ModelAdmin):
