@@ -1,21 +1,14 @@
 from django import forms
 from django.forms import ModelForm
-from .models import Activity, Actor
+from .models import  Actor
 from utilities.forms import BootstrapMixin
 
 BLANK_CHOICE = (("", "---------"),)
 
-
-class ActivityForm(ModelForm):
-    class Meta:
-        model = Activity
-        fields = ['type', 'actor', 'when', 'description']
-
-
-class ActorForm(ModelForm):
+class ActorForm(BootstrapMixin, ModelForm):
     class Meta:
         model = Actor
-        fields = ['name', 'telephone', 'cellphone', 'email']
+        fields = ['name', 'category', 'telephone', 'cellphone', 'email']
 
 
 class ActorFilterForm(BootstrapMixin, forms.ModelForm):
@@ -28,4 +21,4 @@ class ActorFilterForm(BootstrapMixin, forms.ModelForm):
         model = Actor
         fields = [
             'q',
-        ]
+        ]
\ No newline at end of file
