@@ -4,7 +4,7 @@ from simple_history.models import HistoricalRecords
 from django.db import models
 from django.db.models.deletion import CASCADE, PROTECT
 
-from dcim.models import Device
+from dcim.models import Device, Site
 
 class ActorCategory(models.Model):
     name = models.CharField(
@@ -19,42 +19,64 @@ class ActorCategory(models.Model):
     def __str__(self):
         return self.name
 
-
 class Actor(models.Model):
     name = models.CharField(
         max_length=100,
         verbose_name='Complete Name',
     )
+
     email = models.EmailField(
         null=True,
         blank=True,
         verbose_name='E-mail',
     )
+
     category = models.ForeignKey(
         ActorCategory,
         on_delete=models.PROTECT,
         verbose_name='Category',
     )
+
     cellphone = models.CharField(
         max_length=20,
         null=True,
         blank=True,
         verbose_name='Cellphone',
     )
+
     telephone = models.CharField(
         max_length=20,
         null=True,
         blank=True,
         verbose_name='Telephone',
     )
-    is_active = models.BooleanField(
-        default = True,
-    )
+
     history = HistoricalRecords(
         table_name='conectividadeapp_actor_history',
         custom_model_name=lambda x:f'{x}History',
     )
 
+    # This attribute does not allow to be changed by actor._active = True or actor._active = False
+    # To change this attribute, use the actor.activate() method for True or actor.deactivate() for False
+    # To get this attribute, use the actor.is_active()
+    _active = models.BooleanField(
+        default = True,
+    )
+
+    def is_active(self):
+        '''Returns True if the instance is active and False if the instance is inactive'''
+        return self._active
+
+    def activate(self):
+        '''Changes the status of active to True'''
+        self._active = True
+        self.save()
+
+    def deactivate(self):
+        '''Change the status of active to False'''
+        self._active = False
+        self.save()
+
     def __str__(self):
         return self.name
 
@@ -63,63 +85,17 @@ class Actor(models.Model):
     def first_name(self):
         return self.name.split()[0]
 
-    # If called like 'Actor.change_active()', change 'Actor.active' attribute for True if 'Actor.active' is False and False if 'Actor.active' is True.
-    def change_active(self):
-        self.is_active = not self.is_active
-
     class Meta:
         verbose_name = 'Actor'
         verbose_name_plural = 'Actors'
 
 class OldDevice(models.Model):
-
     name = models.CharField(max_length=100, null=True, blank=True)
     rack = models.CharField(max_length=100, null=True, blank=True)
     site = models.CharField(max_length=100, null=True, blank=True)
     ipv4 = models.CharField(max_length=40, null=True, blank=True)
     ipv6 = models.CharField(max_length=70, null=True, blank=True)
 
-class ActivityAbstract(models.Model):
-
-    device = models.ForeignKey(
-        Device,
-        on_delete=models.PROTECT,
-        null=True,
-        blank=True,
-    )
-    olddevice = models.OneToOneField(
-        OldDevice,
-        on_delete=CASCADE,
-        null=True,
-        blank=True,
-    )
-    actor = models.ManyToManyField(
-        Actor,
-    )
-    when = models.DateField(
-        default=date.today,
-    )
-    description = models.TextField(
-        max_length=255,
-        null=True,
-        blank=True,
-    )
-    is_active = models.BooleanField(
-        default = True,
-    )
-
-    def __str__(self):
-        return f'{self.id}'
-
-    def activate(self):
-        self.is_active = True
-
-    def deactivate(self):
-        self.is_active = False
-
-    class Meta:
-        abstract = True
-
 class ActivityReason(models.Model):
     CHOICES = [
         ('INSTALL', 'Instalação'),
@@ -129,106 +105,127 @@ class ActivityReason(models.Model):
     name = models.CharField(
         max_length=256,
     )
+
     type = models.CharField(
         max_length=7,
         choices=CHOICES,
         default='INSTALL',
     )
 
+    # This attribute does not allow to be changed by reason._active = True or reason._active = False
+    # To change this attribute, use the reason.activate() method for True or reason.deactivate() for False
+    # To get this attribute, use the reason.is_active()
+    _active = models.BooleanField(
+        default = True,
+    )
+
+    def is_active(self):
+        '''Returns True if the instance is active and False if the instance is inactive'''
+        return self._active
+
+    def activate(self):
+        '''Changes the status of active to True'''
+        self._active = True
+        self.save()
+
+    def deactivate(self):
+        '''Change the status of active to False'''
+        self._active = False
+        self.save()
+
     def __str__(self):
-        return f'{self.name}'
+        return f'{self.type} - {self.name}'
 
+class Activity(models.Model):
+    device = models.ForeignKey(
+        Device,
+        on_delete=models.PROTECT,
+        null=True,
+        blank=True,
+    )
 
-class Activity(ActivityAbstract):
-        reason = models.ForeignKey(
-            'ActivityReason',
-            on_delete=models.CASCADE,
-            null=True,
-            blank=True,
-        )
-        type = models.CharField(
-            max_length=7,
-            default='INSTALL',
-        )
-        history = HistoricalRecords(
-            table_name='conectividadeapp_activity_history',
-            custom_model_name=lambda x:f'{x}History',
-        )
+    olddevice = models.OneToOneField(
+        OldDevice,
+        on_delete=CASCADE,
+        null=True,
+        blank=True,
+    )
+
+    actor = models.ManyToManyField(
+        Actor,
+    )
+
+    when = models.DateField(
+        default=date.today,
+    )
+
+    description = models.TextField(
+        max_length=255,
+        null=True,
+        blank=True,
+    )
 
-class ActivityInstall(ActivityAbstract):
     reason = models.ForeignKey(
-        ActivityReason,
-        on_delete=models.CASCADE,
-        limit_choices_to={'type': 'INSTALL'}
+        'ActivityReason',
+        on_delete=models.PROTECT,
     )
 
-    type = models.CharField(
-        max_length=7,
-        default='INSTALL'
+    history = HistoricalRecords(
+        table_name='conectividadeapp_activity_history',
+        custom_model_name=lambda x:f'{x}History',
     )
 
-    main_activity = models.ForeignKey(
-        Activity,
-        on_delete=models.CASCADE,
-        null=True, blank=True
+    # This attribute does not allow to be changed by activity._active = True or activity._active = False
+    # To change this attribute, use the activity.activate() method for True or activity.deactivate() for False
+    # To get this attribute, use the activity.is_active()
+    _active = models.BooleanField(
+        default = True,
     )
 
-class ActivityRemove(ActivityAbstract):
-    reason = models.ForeignKey(
-        ActivityReason,
-        on_delete=models.CASCADE,
-        limit_choices_to={'type': 'REMOVE'}
-    )
+    @property
+    def type(self):
+        return self.reason.type
 
-    type = models.CharField(
-        max_length=7,
-        default='REMOVE'
-    )
+    def save_without_historical_record(self, *args, **kwargs):
+        self.skip_history_when_saving = True
+        try:
+            ret = self.save(*args, **kwargs)
+        finally:
+            del self.skip_history_when_saving
+        return ret
 
-    main_activity = models.ForeignKey(
-        Activity,
-        on_delete=models.CASCADE,
-        null=True, blank=True
-    )
+    def is_active(self):
+        '''Returns True if the instance is active and False if the instance is inactive'''
+        return self._active
 
+    def activate(self):
+        '''Change the status of active to True'''
+        self._active = True
+
+    def deactivate(self):
+        '''Change the status of active to False'''
+        self._active = False
+
+    def __str__(self):
+        return f'{self.id}'
 
 '''
 DJANGO SIGNALS FOR ACTIVITY MANAGEMENT
 '''
 
-from django.db.models.signals import post_save, m2m_changed
+from django.db.models.signals import post_save
 from django.dispatch import receiver
 
-@receiver(post_save, sender=ActivityInstall or ActivityRemove)
-def create_new_activity(sender, instance, created, **kwargs):
-    if created:
-        activity = Activity.objects.create(
-            device=instance.device,
-            olddevice=instance.olddevice,
-            when=instance.when,
-            description=instance.description,
-            reason=instance.reason,
-            type=instance.type,
-            is_active=instance.is_active,
+@receiver(post_save, sender=Activity)
+def set_technical_reservation_to_device_site(sender, instance, created, **kwargs):
+    if created and instance.type == 'REMOVE':
+        site, created = Site.objects.get_or_create(
+            name='Technical Reservation',
+            defaults={'slug': 'technical-reservation', 'status': 'active'}
         )
-        instance.main_activity = activity
-        instance.save()
-    else:
-        if not instance.is_active:
-            instance.delete()
+        device = instance.device
+        device.site = site
+        device.rack = None
+        device.save()
 
-post_save.connect(create_new_activity, sender=ActivityInstall)
-post_save.connect(create_new_activity, sender=ActivityRemove)
-
-@receiver(m2m_changed, sender=ActivityInstall.actor.through or ActivityRemove.actor.through)
-def add_actors_to_activity(sender, instance, action, reverse, model, pk_set, **kwargs):
-    if action == 'post_add':
-        main_activity = instance.main_activity
-        actors = model.objects.filter(pk__in=pk_set)
-        for actor in actors:
-            main_activity.actor.add(actor.pk)
-        main_activity.save()
-
-
-m2m_changed.connect(add_actors_to_activity, sender=ActivityInstall.actor.through)
-m2m_changed.connect(add_actors_to_activity, sender=ActivityRemove.actor.through)
+post_save.connect(set_technical_reservation_to_device_site, sender=Activity)
