@@ -15,22 +15,23 @@ menu_items = (
             PluginMenuButton(
                 link='plugins:conectividadeapp:addactor',
                 title='Add a new actor',
-                icon_class='fa fa-plus',
+                icon_class='mdi mdi-plus-thick',
                 color=ButtonColorChoices.GREEN,
-                permissions=['conectividadeapp.view_actor']
+                permissions=['conectividadeapp.add_actor']
             ),
         )
     ),
     PluginMenuItem(
         link='plugins:conectividadeapp:activity_op_list',
         link_text='Activities',
-        permissions=[],
+        permissions=['conectividadeapp.view_activity'],
         buttons=(
              # Add a default button which links to the random animal view
             PluginMenuButton(
                 link='plugins:conectividadeapp:searchdevice',
                 title='searchdevice and activity',
-                icon_class='fa fa-question'
+                icon_class='mdi mdi-magnify',
+                permissions=['conectividadeapp.view_activity']
             ),
 
             # Add a green button which links to the admin view to add a new animal. This
@@ -38,11 +39,10 @@ menu_items = (
             PluginMenuButton(
                 link='plugins:conectividadeapp:list',
                 title='Add a new activity for the last device',
-                icon_class='fa fa-plus',
+                icon_class='mdi mdi-plus-thick',
                 color=ButtonColorChoices.GREEN,
-                permissions=[]
-            ),
+                permissions=['conectividadeapp.add_activity']
             ),
+        ),
     ),
-    
 )
\ No newline at end of file
